#include <msp430.h>

// LCD Segment Mapping
const unsigned char LCD_Char_Map[] =
{
    BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7,                // '0' or 'O'
    BIT5 | BIT6,                                            // '1' or 'I'
    BIT0 | BIT1 | BIT3 | BIT4 | BIT6 | BIT7,                // '2' or 'Z'
    BIT0 | BIT1 | BIT4 | BIT5 | BIT6 | BIT7,                // '3'
    BIT0 | BIT1 | BIT2 | BIT5 | BIT6,                       // '4' or 'y'
    BIT0 | BIT1 | BIT2 | BIT4 | BIT5 | BIT7,                // '5' or 'S'
    BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT7,         // '6' or 'b'
    BIT5 | BIT6 | BIT7,                                     // '7'
    BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7,  // '8' or 'B'
    BIT0 | BIT1 | BIT2 | BIT4 | BIT5 | BIT6 | BIT7,         // '9' or 'g'
};

void clock_init()
{
  // Setup J1.0 SMCLK, J1.1 MCLK, J1.3 ACLK
//  PJDIR |= BIT0 | BIT1 | BIT3;           // ACLK, MCLK, SMCLK set out to pins
//  PJSEL |= BIT0 | BIT1 | BIT3;           // PJ.0,1,3 for debugging purposes.
  
  // Setup LFXT1
  UCSCTL6 &= ~(XT1OFF);                  // XT1 On
  UCSCTL6 |= XCAP_3;                     // Internal load cap
  
  // Loop until XT1 fault flag is cleared
  do
  {
    UCSCTL7 &= ~XT1LFOFFG;             // Clear XT1 fault flags
  } while (UCSCTL7 & XT1LFOFFG);         // Test XT1 fault flag
  
  UCSCTL6 &= ~(XT1DRIVE_3);               // XT1 stable, reduce drive strength
  UCSCTL4 &= ~(SELA0 | SELA1 | SELA2);    // Ensure XT1 is ACLK source
  
  // Initialize DCO to 2.45MHz
  __bis_SR_register(SCG0);               // Disable the FLL control loop
  UCSCTL0 = 0x0000;                      // Set lowest possible DCOx, MODx
  UCSCTL1 = DCORSEL_5;                   // Set RSELx for DCO = 16 MHz
  UCSCTL2 = FLLD_1 | 243;                 // Set DCO Multiplier for 8MHz
  // (N + 1) * FLLRef = Fdco
  // (243 + 1) * 32768 = 8MHz
  // Set FLL Div = fDCOCLK/2
  __bic_SR_register(SCG0);               // Enable the FLL control loop
  
  // Worst-case settling time for the DCO when the DCO range bits have been
  // changed is n x 32 x 32 x f_MCLK / f_FLL_reference. See UCS chapter in 5xx
  // UG for optimization.
  // 32 x 32 x 8 MHz / 32,768 Hz = 250000 = MCLK cycles for DCO to settle
  __delay_cycles(250000);
  
  // Loop until XT1, XT2 & DCO fault flag is cleared
  do
  {
    UCSCTL7 &= ~(XT2OFFG | XT1LFOFFG | DCOFFG);
    // Clear XT2,XT1,DCO fault flags
    SFRIFG1 &= ~OFIFG;                 // Clear fault flags
  } while (SFRIFG1 & OFIFG);             // Test oscillator fault flag
}

int main(void)
{
    volatile unsigned char i;

    WDTCTL = WDTPW | WDTHOLD;               // Stop WDT

    clock_init();
    // Port Configuration
    P1OUT = 0x00; P2OUT = 0x00; P3OUT = 0x00; P4OUT = 0x00; P5OUT = 0x00; P6OUT = 0x00;
    P7OUT = 0x00; P8OUT = 0x00; P9OUT = 0x00; PJOUT = 0x00;
    P1DIR = 0xFF; P2DIR = 0xFF; P3DIR = 0xFF; P4DIR = 0xFF; P5DIR = 0xFF; P6DIR = 0xFF;
    P7DIR = 0xFF; P8DIR = 0xFF; P9DIR = 0xFF; PJDIR = 0xFF;
    
    // Disable SVS
    PMMCTL0_H = PMMPW_H;                    // PMM Password
    SVSMHCTL &= ~(SVMHE | SVSHE);           // Disable High side SVS
    SVSMLCTL &= ~(SVMLE | SVSLE);           // Disable Low side SVS
    
    // Setup LCD_C
    // LCD_FREQ = ACLK/32/4, LCD Mux 4, turn on LCD
    LCDCCTL0 =  LCDDIV_28 | LCDPRE_2 | LCD4MUX | LCDON;
    
//    LCDCCTL0 |= LCDDIV_28 | LCDPRE_2 | LCD4MUX | LCDSON | LCDLP;
    //Charge pump generated internally at 2.96V, external bias (V2-V4) generation
    //Internal reference for charge pump
    LCDCVCTL = LCDCPEN | VLCD_2_96;
    REFCTL0 &= ~REFMSTR;

    LCDCPCTL0 = 0x003F;
    LCDCPCTL1 = 0x0000;
//    LCDCPCTL0 = 0x0030;                     //Select LCD Segments 4-5
//    LCDCPCTL1 = 0x0000;                     //

    while (1)
    {
        for (i = 0; i < sizeof(LCD_Char_Map); i++)
        {
            LCDM3 &= ~LCD_Char_Map[8];      // Clear LCD
            LCDM3 = LCD_Char_Map[i];        // Display character
            __delay_cycles(500000);
        }
        LPM3;
    }
}